/**
 * The PaddedLeatherShirt class represents a leather shirt that can be worn by the kobold.
 */
function PaddedLeatherShirt() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Padded Leather Shirt";
    var itemDescription = `A basic piece of armor, often worn by archers and other backline soldiers. A favorite amongst brigands and bandits.`;
    var baseValue = 60;
    var offensiveValue = 0;
    var defensiveValue = 6;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.PADDEDLEATHERSHIRT;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.SHIRT;


    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLocaleLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 220px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };
    
    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the groin, and false otherwise.
     * @returns {boolean} True if the shirt covers the groin, and false otherwise.
     */
    this.coversGroin = function () {
        return false;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the rear, and false otherwise.
     * @returns {boolean} True if the shirt covers the rear, and false otherwise.
     */
    this.coversRear = function () {
        return false;
    };
    
    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the chest, and false otherwise.
     * @returns {boolean} True if the shirt covers the chest, and false otherwise.
     */
    this.coversChest = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt can be torn, and false otherwise.
     * @returns {boolean} True if the shirt can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt is a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shirt is a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return global.kobold.body.getBaseGender() !== GenderEnum.MALE; // Leather armor is for boys.
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {PaddedLeatherShirt-stripped} shirt A PaddedLeatherShirt object, stripped of private members and all functions.
     */
    this.load = function (shirt) {
        
    };
}
