/**
 * This class represents a single change for use with the ChangeNotificationBuilder.
 * @param {string} why Why did the change occur?
 * @param {string} what What changed? Stamina, agility, health?
 * @param {int} amount By how much did it change?
 * @param {boolean} positiveBad Inverts positive and negative value display if set to true.
 */
function Change(why, what, amount, positiveBad) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.reason = why;
    this.stat = what;
    this.value = amount;
    this.invert = positiveBad;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Transforms this change into valid HTML.
     * @returns {string} HTML describing this change.
     */
    this.toHtml = function () {
        var abs = Math.abs(that.value);
        var span;
        if (!that.invert) {
            span = that.value < 0 ? `<span class="warningtext bold">` : `<span>`;
        } else {
            span = that.value >= 0 ? `<span class="warningtext bold">` : `<span>`;
            
        }
        var number = that.value < 0 ? ` -${abs}` : ` +${abs}`;

        return `<div class="text nomargin center">${that.reason}: ${span}${that.stat}${number}</span></div>`;
    };
}
