/**
 * Unknown section of the forest scene.
 * @param {ForestController} controller The forest controller.
 */
function ForestUnknownPry(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var clawId = "unknownPryClawButton";
    var biteId = "unknownPryInitialBiteButton";
    var stillId = "unknownPryStillButton";

    var toClaw = new ForestUnknownClaw(ctrl);
    var toBite = new ForestUnknownInitialBite(ctrl);
    var toStill = new ForestUnknownStayStill(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You approach the spiky plant, taking care [i]not[/i] to step on any low-hanging tendrils with your bare feet.
                            Your shirt, hanging off the side of the decently-sized shrub, is impaled by several of its organic barbs, 
                            all the way from the end of one sleeve to the very collar.
                            Ignoring the fact that you don't quite recall getting entangled in the thornbush [i]that[/i] badly,
                            you deftly get to work freeing your garment from nature's fabric-ruining grasp.
                        [/p]
                        [p]
                            You painstakingly unhook your sweater from the myriad of thorns it is caught on, one at a time.
                            Luckily, each spike individually isn't all that hard to remove.
                            Due to the lack of jagged edges, you can simply pluck the thick woolen cloth away from the brambles without damaging it all too much.
                        [/p]
                        [p]
                            After a solid few minutes of concentrated effort, you've managed to completely remove your shirt from the bush.
                            Satisfied with your work, you hold the garment up to the sky with both your hands, 
                            one of the few rays of sunlight beaming through the canopy illuminating the sweater just enough for you to inspect it a bit more closely.
                            Apart from a few, hardly noticeable pinpoint pricks marking where the thorns used to be, it seems to be in perfect condition.
                        [/p]
                        [p]
                            With your gaze directed at your white woolen sweater, and with glee washing over your mind,
                            you fail to notice the many tendrils surrounding you slowly coming to life.
                            Just as you were about to proclaim this your [i]luckiest day [b]ever[/b][/i],
                            one of the thorny tentacles stretches out towards you, uncoiling like a snake leaping at its prey.
                            Caught completely off guard, your heart skips a beat as the surprisingly flexible, 
                            spike-covered plant part wraps itself around one of your wrists in the flash of an eye.
                        [/p]
                        [p]
                            You let out an audible, frightened gasp, the shirt falling from your hands onto the ground out of sheer astonishment.
                            Your first reflex is to yank your arm back, but this causes the thorns covering the tendril to press painfully against your scales.
                            You only barely manage to suppress your natural bodily response before drawing your own blood,
                            the tentacle completely unwavering in its determination to hold your arm in place.
                            A mixture of pain and fear causes adrenaline to rapidly flood your system, and time itself seems to slow down.
                            Even then, it takes your brain a few seconds to [b]process[/b] what exactly just happened.
                        [/p]
                        [p]
                            The thornbush seems to be [i]alive[/i], and intent on holding you in place.
                            Besides the single tendril already tightly wrapped around your wrist,
                            the ones on the ground surrounding you are creeping towards your feet, undoubtebly to further restrain you.
                            You've no clue as to what the plant's intentions are, but judging by the painful grip it has on your arm, 
                            you'd guess that it either wants to [b]eat you[/b] or...[i]do something else[/i].
                            You're not sure if you like either of those options.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the ant choice button.
     */
    function buildClawChoice() {
        var disabled = ctrl.unknown.pry.base.has() ? "disabled" : "";
        var isSelected = ctrl.unknown.pry.base.get() === ForestUnknownPryChoice.CLAW;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${clawId}" ${disabled}>Swipe</button>`;

        var tooltip = `Use the sharp claws of your free hand in an attempt to cut the tendril clutching your arm.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 175px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the shake choice button.
     */
    function buildBiteChoice() {
        var disabled = ctrl.unknown.pry.base.has() ? "disabled" : "";
        var isSelected = ctrl.unknown.pry.base.get() === ForestUnknownPryChoice.BITE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${biteId}" ${disabled}>Bite</button>`;

        var tooltip = `Attempt to sever the thorny tentacle using your pointed teeth.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }
    
    /**
     * Builds and returns the shake choice button.
     */
    function buildStillChoice() {
        var disabled = ctrl.unknown.pry.base.has() ? "disabled" : "";
        var isSelected = ctrl.unknown.pry.base.get() === ForestUnknownPryChoice.STILL;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${stillId}" ${disabled}>Stay still</button>`;

        var tooltip = `Stay perfectly still.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 140px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildClawChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildBiteChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildStillChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + clawId), $("#" + biteId), $("#" + stillId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerClawListener() {
        $("#" + clawId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unknown.pry.base.set(ForestUnknownPryChoice.CLAW);
            loadNextPart(toClaw);
        });

    }

    /**
     * Adds a listener to the button.
     */
    function registerBiteListener() {
        $("#" + biteId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unknown.pry.base.set(ForestUnknownPryChoice.BITE);
            loadNextPart(toBite);
        });
    }
    
    /**
     * Adds a listener to the button.
     */
    function registerStillListener() {
        $("#" + stillId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unknown.pry.base.set(ForestUnknownPryChoice.STILL);
            loadNextPart(toStill);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "forestUnknownPry";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.unknown.base.get() !== ForestUnknownChoice.PRY) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.unknown.pry.base.change();
        part += buildChoice();
        part += toClaw.build();
        part += toBite.build();
        part += toStill.build();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.unknown.base.get() !== ForestUnknownChoice.PRY) {
            return;
        }

        toClaw.registerListeners();
        toBite.registerListeners();
        toStill.registerListeners();

        if (ctrl.unknown.pry.base.has()) {
            return;
        }

        registerClawListener();
        registerBiteListener();
        registerStillListener();
    };
}

var ForestUnknownPryChoice = {
    CLAW: 1,
    BITE: 2,
    STILL: 3
};
