/**
 * Unknown section of the forest scene.
 * @param {ForestController} controller The forest controller.
 */
function ForestUnknownYank(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var grabId = "unknownYankGrabButton";
    var hitId = "unknownYankHitButton";
    var dodgeId = "unknownYankDodgeButton";

    var toGrab = new ForestUnknownGrab(ctrl);
    var toHit = new ForestUnknownHit(ctrl);
    var toDodge = new ForestUnknownDodge(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You move a bit closer towards the spiky shrub, making sure not to step on any lowhanging, barb-covered tendrils.
                            Your shirt is thoroughly [b]impaled[/b] on a multitude of thorns.
                            Far too many for you to carefully remove.
                            You [i]could[/i] try to gently pry the fabric apart the plant, but the situation seems rather hopeless.
                            Besides, it'd take precious time, time that could be better spent evading whoever may be chasing you.
                        [/p]
                        [p]
                            Instead of setting yourself up for failure, you decide to go for the quick and easy route.
                            You take hold of the stuck garment with both your hands, and then fiercely [b]pull[/b] it away from the plant's exterior.
                            A loud tearing noise echoes throughout the small clearing, as one particularly well-punctured part simply [b]refuses[/b] to budge.
                            While you were hoping to retrieve the sweater with perhaps a few small holes in it,
                            you accidentally completely [b]shred[/b] the old, worn fabric to pieces in your attempt to yank it off of the thornbush.
                        [/p]
                        [p]
                            When you realize what you just did, it is already too late.
                            The deed has been done.
                            Plucks of what used to be your favorite piece of clothing stick to the thorny bush, 
                            while a large strip of now-useless fabric flutters to the ground.
                            You have inadvertently [b]destroyed[/b] one of your most precious possession, worth ten times the gold around your neck in sentimental value.
                        [/p]
                        [p]
                            Stupid. Stupid! [b]Stupid![/b]
                            How could you have done this?!
                            You should have [b]thought[/b] before you [b]acted[/b]!
                            You nearly start to cry, but manage to withhold your tears, for now.
                        [/p]
                        [p]
                            You hold the ${global.kobold.inventory.createItemWithTooltip(new RippedWoolSweater())} up to the sky,
                            one of the sparse rays of light beaming through the canopy illuminating it, allowing you to properly inspect the results of your actions.
                            The backside is practically torn in half.
                            Large pieces of cloth were ripped from the front.
                            One of the sleeves is hanging on by a thread, and the other has a few large holes in it.
                        [/p]
                        [p]
                            The result of your rash move is a bunch of rags that are barely strung together,
                            although you guess you could still wear what [i]used to be[/i] your [i]shirt[/i], 
                            if just until the fabric gives way completely.
                            As a memento.
                            A keepsake.
                            A reminder to always [b]think[/b] before you [b]do[/b].
                        [/p]
                        [p]
                            As you continue to lament the loss of your most prized belonging, 
                            you suddenly see something [i]moving[/i] through one of the many holes lining your now-ruined shirt.
                            Lowering the fabric covering your view, you bare witness to the plant's many spiked tendrils starting to move about.
                        [/p]
                        [p]
                            At first, you can't believe your eyes.
                            You think you're hallucinating, that your mind is unable to cope with the loss of your shirt,
                            and is thus imagining some sort of [i]boogeyman[/i], but the longer you stare at the slithering mass of thorned tentacles in front of you, 
                            the more you realize that you [i]aren't[/i] dreaming.
                        [/p]
                        [p]
                            You decide to keep your guard up.
                            Most of the tentacles are inching towards you.
                            You don't know the intentions of this [i]thing[/i] you seem to have awoken,
                            but if whatever it has planned involves those prickly tendrils, then you doubt you want to be a part of it.
                            Your instincts tell you to flee, but your rational mind reasons that, if you [i]do[/i] turn tail and run,
                            you might simply be presenting the monster with an easier target.
                        [/p]
                        [p]
                            Instead of outright bolting, you slowly begin to back away from the monstrosity, its tentacles menacingly advancing upon you.
                            One of them seemingly coils up, like a snake preparing to strike.
                            Then, just like the predator it's mimicking, the thorny appendage [b]leaps[/b] from the ground, lunging towards you at high speed.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the ant choice button.
     */
    function buildGrabChoice() {
        var disabled = ctrl.unknown.yank.base.has() ? "disabled" : "";
        var isSelected = ctrl.unknown.yank.base.get() === ForestUnknownYankChoice.GRAB;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${grabId}" ${disabled}>Grab it</button>`;

        var tooltip = `Grab the tendril before it can grab you.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 170px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the shake choice button.
     */
    function buildHitChoice() {
        var disabled = ctrl.unknown.yank.base.has() ? "disabled" : "";
        var isSelected = ctrl.unknown.yank.base.get() === ForestUnknownYankChoice.HIT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${hitId}" ${disabled}>Hit it</button>`;

        var weapon = ctrl.unknown.weapon();
        var tooltip;
        
        if (weapon) {
            tooltip = `Bash the tentacle with your ${weapon.name()}.`;
        } else {
            tooltip = `Punch the tentacle.`;
        }
        
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the shake choice button.
     */
    function buildDodgeChoice() {
        var disabled = ctrl.unknown.yank.base.has() ? "disabled" : "";
        var isSelected = ctrl.unknown.yank.base.get() === ForestUnknownYankChoice.DODGE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${dodgeId}" ${disabled}>Dodge it</button>`;

        var tooltip = `Dodge the lunging tendril.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 110px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildGrabChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildHitChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildDodgeChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + grabId), $("#" + hitId), $("#" + dodgeId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerGrabListener() {
        $("#" + grabId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unknown.yank.base.set(ForestUnknownYankChoice.GRAB);
            loadNextPart(toGrab);
        });

    }

    /**
     * Adds a listener to the button.
     */
    function registerHitListener() {
        $("#" + hitId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unknown.yank.base.set(ForestUnknownYankChoice.HIT);
            loadNextPart(toHit);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerDodgeListener() {
        $("#" + dodgeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unknown.yank.base.set(ForestUnknownYankChoice.DODGE);
            loadNextPart(toDodge);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "forestUnknownYank";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.unknown.base.get() !== ForestUnknownChoice.YANK) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.unknown.yank.base.change();
        part += buildChoice();
        part += toGrab.build();
        part += toHit.build();
        part += toDodge.build();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.unknown.base.get() !== ForestUnknownChoice.YANK) {
            return;
        }

        toGrab.registerListeners();
        toHit.registerListeners();
        toDodge.registerListeners();

        if (ctrl.unknown.yank.base.has()) {
            return;
        }

        registerGrabListener();
        registerHitListener();
        registerDodgeListener();
    };
}

var ForestUnknownYankChoice = {
    GRAB: 1,
    HIT: 2,
    DODGE: 3
};
