/**
 * The hide part of the woods section of the Forest scene.
 * @param {ForestController} controller The forest controller.
 */
function ForestWoodsHide(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var content = `<div class="text">Hide</div>`;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the main content of this section.
     * @returns {string} The main content of this section.
     */
    function buildContent() {
        return content;
    }


    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        //choice += buildUnknownChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        //choice += buildHideChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        //choice += buildKnownChoice();
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + unknownId), $("#" + hideId), $("#" + knownId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {ForestWoodsArrivalChoiceEnum} next Which part to load.
     */
    function loadNextPart(next) {
        var part;
        if (next === ForestWoodsArrivalChoiceEnum.UNKNOWN) {
            part = unknown;
        }
        if (next === ForestWoodsArrivalChoiceEnum.HIDE) {
            part = hide;
        }
        if (next === ForestWoodsArrivalChoiceEnum.KNOWN) {
            part = known;
        }
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the known button.
     */
    function registerKnownListener() {
        $("#" + knownId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.woods.makeArrivalChoice(ForestWoodsArrivalChoiceEnum.KNOWN);
            loadNextPart(ForestHomePathChoiceEnum.CONTINUE);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "woodsHide";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.woods.hasMadeArrivalChoice()) {
            return "";
        }
        if (ctrl.woods.getArrivalChoice() !== ForestWoodsArrivalChoiceEnum.HIDE) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.woods.hasMadeArrivalChoice()) {
            return;
        }
        if (ctrl.woods.getArrivalChoice() !== ForestWoodsArrivalChoiceEnum.HIDE) {
            return;
        }
        if (ctrl.woods.hasMadeHideChoice()) {
            return;
        }
        // Register listeners here
    };
}
