/**
 * Not barn section of the farm scene.
 * @param {BreedingFarmController} controller The breeding farm controller.
 */
function NotBarnAttack(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var lootId = "notBarnAttackLootButton";
    var freeId = "notBarnAttackFreeButton";

    var toLoot = new NotBarnLoot(ctrl);
    var toFree = new NotBarnFree(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon(); // TODO
        var toReturn = `[p]
                            No.
                            [b]No![/b]
                            What Cassidy is doing is [b]wrong[/b], no matter how you look at it.
                            She's [b]evil[/b].
                            Anger wells up inside of you.
                            You wield your ${global.kobold.inventory.createItemWithTooltip(weapon)} and approach the vulpine from behind.
                            She's too busy fumbling with the door to notice you drawing close.
                            You're not going to let this horrible [b]bitch[/b] get away with [b]enslaving[/b] your kind.
                        [/p]`;
        
        if(weapon.stab()) {
            global.kobold.history.shared.cassidy.dead = true;
            toReturn += `[p]
                            In a single smooth motion, you grab the vixen's waist with one arm, whilst driving your ${global.kobold.inventory.createItemWithTooltip(weapon)}
                            into her side with the other.
                            Like embracing her in a hug from behind, one almost as sick and twisted as she herself is.
                            The blade of your weapon slides in between her ribs, sinking into her delicate flesh all the way to the hilt.
                            Your rage surging, you stab her, again and again and again.
                        [/p]
                        [p]
                            She doesn't scream.
                            She doesn't struggle.
                            Instead, she solemnly closes her eyes, and lowers her head.
                            It's like she knew this was coming.
                            Like she'd long made peace with the fact that she's a horrible person.
                            Like she accepts, or perhaps even [i]welcomes[/i] this outcome to her wicked life.
                            Or maybe her mind just can't process what is happening.
                        [/p]
                        [p]
                            As your series of stabs comes to an end, blood [b]gushing[/b] from her side,
                            Cassidy takes one last deep breath, before her still-standing figure lifelessly topples forwards.
                            The vulpine's naked frame tumbles against the door, her arm impacting the door handle, causing it to swing open.
                            Her body ends up halfway inside of her own mansion, legs and rump laying on the porch, 
                            a large pool of blood forming on the indoors floor, ending up leaking from the doorway.
                            [b]You've killed Cassidy.[/b]
                        [/p]
                        [p]
                            You take a few deep breaths, excitement rushing from your system.
                            You acted on impulse, there's no denying that.
                            But she said it herself.
                            She breeds kobolds to be [b]slaughtered[/b].
                            To be [b]eaten[/b].
                            You couldn't let her get away with that.
                            Kobolds don't have any rights.
                            The guards wouldn't have stopped her.
                            You did what you had to.
                            Or so you tell yourself, to dull your aching conscience.
                        [/p]
                        [p]
                            In the back of your mind, you realize you've crossed a terrible threshold,
                            but the adrenaline coursing through your veins makes you ignore the crushing guilt building up inside of you, at least for now.
                            You glance around.
                            You don't think anyone saw you, but it's only a matter of time before someone notices the open door and the corpse in the doorway.
                            Whatever you decide to do, you'll have to act fast.
                        [/p]`;
        } else {
            toReturn += `[p]
                            You brace your weapon, holding it high above your head, preparing your entire body to strike down with all of your might.
                            You fail to contain a loud, angry growl, alerting the vixen to your presence directly behind her, yet before she even manages to react,
                            you bring your ${global.kobold.inventory.createItemWithTooltip(weapon)} down upon her head,
                            the hard surface impacting against the woman's skull with a twistingly satisfying [b]crack[/b].
                        [/p]
                        [p]
                            The vulpine's naked frame tumbles against the door, her arm impacting the door handle, causing it to swing open.
                            Her body ends up halfway inside of her own mansion, legs and rump laying on the porch, 
                            For a moment, you fear you might have [b]killed[/b] her, but then you notice the vixen's bare, 
                            prone torso slowly, yet rhythmically inflating and deflating.
                            Still breathing.
                            You didn't kill her, you've just managed to knock her unconscious in a single blow.
                        [/p]
                        [p]
                            Content with your work, you take a few deep breaths to calm yourself down.
                            That'll teach her.
                            You acted on impulse, there's no denying that.
                            But she said it herself.
                            She breeds kobolds to be [b]slaughtered[/b].
                            To be [b]eaten[/b].
                            You couldn't let her get away with that.
                        [/p]
                        [p]
                            You glance around.
                            You don't think anyone saw you, but it's only a matter of time before someone notices the open door and the passed out vixen in the doorway.
                            Or worse yet: she might get up herself.
                            Whatever you decide to do, you'll have to act fast.
                        [/p]`;
        }
        
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildLootChoice() {
        var disabled = ctrl.postnotbarn.has() ? "disabled" : "";
        var isSelected = ctrl.postnotbarn.get() === NotBarnAttackChoice.LOOT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${lootId}" ${disabled}>Steal</button>`;
        var tooltip = global.parser.parse(`Enter the mansion and take whatever you can.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;
        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFreeChoice() {
        var disabled = ctrl.postnotbarn.has() ? "disabled" : "";
        var isSelected = ctrl.postnotbarn.get() === NotBarnAttackChoice.FREE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${freeId}" ${disabled} style="width:200px;">Free the kobolds</button>`;
        var tooltip = global.parser.parse(`Go back to the barn and set the vixen's captives free.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        button = `<div class="inlinechoice" style="width:200px;">`;
        button += popupContainer;
        button += `</div>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildLootChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildFreeChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + lootId), $("#" + freeId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerLootListener() {
        $("#" + lootId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.postnotbarn.set(NotBarnAttackChoice.LOOT);
            loadNextPart(toLoot);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerFreeListener() {
        $("#" + freeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.postnotbarn.set(NotBarnAttackChoice.FREE);
            loadNextPart(toFree);
        });
    }


    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "notBarnAttack";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.notbarn.get() !== NotBarnTeaDoorChoice.ATTACK && ctrl.notbarn.get() !== NotBarnChoice.ATTACK) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += toLoot.build();
        part += toFree.build();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.notbarn.get() !== NotBarnTeaDoorChoice.ATTACK && ctrl.notbarn.get() !== NotBarnChoice.ATTACK) {
            return;
        }

        toLoot.registerListeners();
        toFree.registerListeners();

        if (ctrl.postnotbarn.has()) {
            return;
        }

        registerLootListener();
        registerFreeListener();
    };
}

var NotBarnAttackChoice = {
    LOOT: 1,
    FREE: 2
};
