/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseShed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var tridentId = arguments.callee.name + "ButtonOne";
    var noWeaponId = arguments.callee.name + "ButtonTwo";

    var toTrident = new FarmerHouseShedTrident(ctrl);
    var toNoWeapon = new FarmerHouseShedNoWeapon(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon();
        var toReturn = `[p]
                            You make the wise decision of [b]not[/b] taking any unnecessary risks.
                            You creep towards the door, quietly open it once more, and slip out of the dilapidated homestead.
                            A second wave of relief is your body's way of thanking you for [b]not[/b] making it have to wither more uncertainty and stress.
                        [/p]
                        [p]
                            You walk towards the shed.
                            Like the back door, it is unlocked, and like the house, it is rickety to the point of barely being able to stand upright.
                            You reckon a firm gust of wind could break it down, but you're no engineer.
                            Like your home, it might be more stable than it appears.
                        [/p]
                        [p]
                            It looks like you were right.
                            Tools, but not just farming tools.
                            A near-blunt saw, a chipped hammer, a bunch of nails, a plow that's seen better days,
                            a spade made out of wood, a hoe, a rake and a pitchfork.
                            Those are the ones you recognize.
                            The rest, you imagine, are various farming and carpenting tools that a dishonest kobold like yourself has no use for.
                        [/p]
                        [p]
                            Darnit.
                            Where's all the good stuff at?!
                            You were hoping to find a scythe or a sickle or at least a pickaxe.
                            The farmer must have taken those.
                            Or maybe he just doesn't own them.
                            Seeing as he doesn't seem too keen on [b]maintaining[/b] what little he [i]does[/i] have,
                            you doubt the [i]quality[/i] of the tools would have been all that great anyway.
                        [/p]
                        [p]
                            The only thing serviceable in this entire shed would be the pitchfork.
                            It's far too big for someone of your [i]posture[/i] to use,
                            but maybe you can [i]trim[/i] the handle somehow.
                            ${weapon ? `You [b]would[/b] have to leave behind your ${global.kobold.inventory.createItemWithTooltip(weapon)} behind, though.
                            No point in carrying two weapons.` : ``}
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildTridentChoice() {
        var disabled = ctrl.shed.has() ? "disabled" : "";
        var isSelected = ctrl.shed.get() === FarmerHouseShedChoice.TRIDENT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${tridentId}" ${disabled}>Take it</button>`;

        var tooltip = global.parser.parse(`With some slight alterations, the pitchfork could make a formidable weapon.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNoWeaponChoice() {
        var disabled = ctrl.shed.has() ? "disabled" : "";
        var isSelected = ctrl.shed.get() === FarmerHouseShedChoice.NOWEAPON;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${noWeaponId}" ${disabled}>Do not</button>`;

        var tooltip = global.parser.parse(`A pitchfork? Hardly a weapon for a thief. What are you, a farmer?`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildTridentChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildNoWeaponChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + tridentId), $("#" + noWeaponId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerTridentListener() {
        $("#" + tridentId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.shed.set(FarmerHouseShedChoice.TRIDENT);
            loadNextPart(toTrident);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerNoWeaponListener() {
        $("#" + noWeaponId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.shed.set(FarmerHouseShedChoice.NOWEAPON);
            loadNextPart(toNoWeapon);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.steal.get() !== FarmerHouseStealChoice.SHED) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toTrident.build();
        part += toNoWeapon.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.steal.get() !== FarmerHouseStealChoice.SHED) {
            return;
        }

        toTrident.registerListeners();
        toNoWeapon.registerListeners();

        if (ctrl.shed.has()) {
            return;
        }

        registerTridentListener();
        registerNoWeaponListener();
    };
}

var FarmerHouseShedChoice = {
    TRIDENT: 1,
    NOWEAPON: 2
};

