/**
 * Buy section of the Workshop scene.
 * @param {SekiWorkshopController} controller The Seki controller.
 */
function SekiWorkshopCenter(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var breedingId = "centerBreedingButton";
    var farmHomeId = "centerFarmHomeButton";
    var wellId = "centerWellButton";
    var varanarId = "centerVaranarButton";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Disheartened, you leave Seki's store and walk towards the small well at the very center of the hamlet.
                            Leaning against it, you take a moment to collect your thoughts.
                        [/p]
                        [p]
                            Seki is a brilliant alchemist.
                            Perhaps even one of the best ones to have ever lived.
                            But to you, he's more than that.
                            He's your kind of, not really, somewhat of a [i]father figure[/i].
                            You wouldn't know what to do if he was gone.
                            You're not ready to lose him.
                        [/p]
                        [p]
                            You can't.
                            You [b]won't[/b].
                            You're [b]not[/b] going to.
                            Even if it means you need to travel through [b]hell[/b] itself to drag his old,
                            feathery ass out of the afterlife,
                            you are [b]not[/b] losing Seki, [b]no matter what[/b].
                            If you need to punch the Grim Reaper himself in the face, then so be it.
                            That bony bastard is [b]not[/b] taking your feathery father from you.
                            [b]Over your dead body.[/b]
                        [/p]
                        [p]
                            You kick a small rock in frustration.
                            It skips along the dirt for a few feet, then comes to rest.
                            You wipe a few tears from your eyes.
                            Luckily, nobody is around to see you.
                            This place looks deserted.
                            Everyone is probably indoors or out working the field or in Varanar or wherever.
                        [/p]
                        [p]
                            You can't sit around here moping all day long.
                            Well, you could, but Seki wouldn't want that.
                            He'd probably chase you from the hamlet himself if he could run for more than five feet without being out of breath.
                            You're going to miss him.
                            A lot.
                            You almost burst into tears.
                            It's just [i]not fair[/i].
                        [/p]
                        [p]
                            Taking a look around, you spot three places of interest that might prove worthy of exploration.
                            The breeding farm across the road from Seki's workshop is an obvious choice.
                            Both the fancy-looking house as well as the attached barn could be full of stuff for you to steal.
                            You don't know [i]who[/i] owns that place, though.
                            You've heard Seki talk about some woman constantly buying up all of his ${global.kobold.inventory.createItemWithTooltip(new FemPlus())}s.
                            Could be related, but you don't really know for certain.
                        [/p]
                        [p]
                            Alternatively, you could break into a random farmer's house.
                            Should be empty, save for a stay-at-home spouse or perhaps some offspring.
                            There probably won't be [i]that much[/i] to steal, but it's a safer bet than the breeding farm.
                        [/p]
                        [p]
                            The final point of interest is the well you're leaning against.
                            It might prove valuable to give it a more [i]thorough[/i] inspection.
                            People around these rural parts are pretty superstitious.
                            You wouldn't be surprised if they hurled an entire fortune worth of individual coins down the waterhole,
                            praying for good luck or wishing for a better life.
                        [/p]
                        [p]
                            You could forego all of these and head to the city directly.
                            You plan on going there after doing whatever it is you plan on doing anyway.
                            And it might help you put your mind off of the poor old owl.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${breedingId}" ${disabled ? "disabled" : ""} style="margin: auto; float: left;">`;
        choice += `Breeding farm &gt;&gt`;
        choice += `</button>`;
        choice += `<button class="button bigbutton continuebutton" id="${farmHomeId}" ${disabled ? "disabled" : ""} style="margin: auto; float: right;">`;
        choice += `Farmer's home &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        choice += `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${wellId}" ${disabled ? "disabled" : ""} style="margin: auto; float: left;">`;
        choice += `Well investigation &gt;&gt`;
        choice += `</button>`;
        choice += `<button class="button bigbutton continuebutton" id="${varanarId}" ${disabled ? "disabled" : ""} style="margin: auto; float: right;">`;
        choice += `To Varanar &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + breedingId), $("#" + farmHomeId), $("#" + wellId), $("#" + varanarId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }
    
    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerBreedingListener() {
        $("#" + breedingId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.breedingSceneName, ctrl.folderName);
        });
    }
    
    /**
     * Adds a listener to the button.
     */
    function registerFarmHouseListener() {
        $("#" + farmHomeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.farmHomeSceneName, ctrl.folderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "sekiCenter";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.free.continued() && !ctrl.buyContinue.continued()) {
            return "";
        }
        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `<span class="todo">You can't go to Varanar or investigate the well.</span>`;
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.free.continued() && !ctrl.buyContinue.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }
        
        registerBreedingListener();
        registerFarmHouseListener();
    };
}
